## Stateful Component
In this example, we are going to create a simple stateful component with a button 
and a message indicating how many times that button has been clicked.

### View
We can start by defining a view:

```javascript
import React from 'react'

function view (props, state) {
  return (
    <div>
      <span>Times clicked: {state.timesClicked}</span>
      <button>Click me</button>
    </div>
  )
}
```

### Actions
In Recycle all actions are defined separately, 
keeping the view "clean" (without `onClick`, `keyUp` or similar event handlers).

Since we need to listen for click events on a button element, 
we can use `sources.DOM` for selecting a DOM element and return an action `{ type: 'buttonClicked' }` every time that button is clicked:

```javascript
function actions (sources) {
  const button = sources.DOM.select('button')

  return [
    button.events('click')
      .mapTo({ type: 'buttonClicked' })
  ]
}

export default actions
```

### Reducers
To update our component we need to change its state, which is done by defining a `reducer`:

```javascript
function reducers (sources) {
  return [
    sources.actions
      .filterByType('buttonClicked')
      .reducer(function (state) {
        state.timesClicked++
        return state
      })
  ]
}

export default reducers
```

### Defining a Recycle component
We now have all the ingredients for defining our component: 

```javascript
import view from './view'
import actions from './actions'
import reducers from './reducers'

function ClickCounter () {
  return {
    initialState: {
      timesClicked: 0
    },
    actions,
    reducers,
    view
  }
}

export default ClickCounter
```

or if you prefer writing it all in a single function:

```javascript
import React from 'react'

function SingleCounter () {
  return {
    initialState: {
      timesClicked: 0
    },

    actions(sources) {
      const button = sources.DOM.select('button')

      return [
        button.events('click')
          .mapTo({ type: 'buttonClicked' })
      ]
    },

    reducers(sources) {
      return [
        sources.actions
          .filterByType('buttonClicked')
          .reducer(function (state) {
            state.timesClicked++
            return state
          })
      ]
    },

    view(props, state) {
      return (
        <div>
          <span>Times clicked: {state.timesClicked}</span>
          <button>Click me</button>
        </div>
      )
    }
  }
}
```
